/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

var log = ko.logging.getLogger("project-debugging-properties");

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var gInvocation = null;
var gInvokeType = Components.interfaces.koIInvocation.INVOKE_DEBUG;
var gInvocationPrefs = null;
var def_invocation_name = 'Project';


//      called prior to any pref widgets getting initialized
function OnPreferencePageInitalize(prefset)
{
    try {
    if (prefset.hasStringPref('lastInvocation')) {
        def_invocation_name = prefset.getStringPref('lastInvocation');
    }
    
    // either get the existing invocation prefs, or create them
    gInvocationPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    if (prefset.hasPref("Invocations")) {
        // we only add this new prefset in onOK handler to avoid dirtying the
        // project with an empty prefset
        gInvocationPrefs.update(prefset.getPref("Invocations"));
    }
    createInitialInvocationSet();
    } catch(e) {
        log.exception(e);
    }
}
    
function createInitialInvocationSet() {
    try {
    // create a project invocation so we can use this to set/get prefs
    gInvocation = Components.classes["@activestate.com/koInvocation;1?type=Project"].createInstance();
    // create a default prefset to use
    var catOb = new Object();
    var instOb = new Object();
    gInvocation.getDefaultPreferences(catOb, instOb);
    gInvocation.currentCategoryPreferences = catOb.value;
    gInvocation.currentInstancePreferences = instOb.value;

    if (gInvocationPrefs.hasPref(def_invocation_name)) {
        // get the existing prefs, and update the invocation with them
        gInvocation.currentInstancePreferences = gInvocationPrefs.getPref(def_invocation_name);
    } else {
        // fresh, no prefs, create them and set them on our prefset
        gInvocation.currentInstancePreferences.id = def_invocation_name;
        gInvocationPrefs.setPref(gInvocation.currentInstancePreferences.id,
                                 gInvocation.currentInstancePreferences);
        // XXX need a default language setting
        gInvocation.currentInstancePreferences.setStringPref("language","Perl");
        if (parent.file)
            gInvocation.currentInstancePreferences.setStringPref("cwd", parent.file.dirName);
    }
    } catch(e) {
        log.exception(e);
    }
}

function OnPreferencePageLoading(prefset)
{
    try {
        var language = gInvocation.currentInstancePreferences.getStringPref("language");
        // set the menu item
        var menu = document.getElementById('language');
        menu.setAttribute('label', language);

        FillInvocationPopup();
        UpdateInvocation();
    } catch(e) {
        log.exception(e);
    }
}

function OnPreferencePageClosing(prefset, ok)
{
//      called when the user clicks ok
    if (!ok) return true;
    try {
        savePrefs(gInvocation.persistableInstancePreferences);
        if (prefset.hasPrefHere("Invocations")) {
            prefset.deletePref("Invocations");
        }
        var configurations = new Object();
        gInvocationPrefs.getPrefIds(configurations, new Object());
        // remove any unset prefs
        for (var i =0; i < configurations.value.length; i++) {
            var prefs = gInvocationPrefs.getPref(configurations.value[i]);
            if (!prefs.hasPref("filename") || !prefs.getStringPref("filename")) {
                gInvocationPrefs.deletePref(configurations.value[i]);
            }
        }
        
        // if there are any configurations left, save them
        gInvocationPrefs.getPrefIds(configurations, new Object());
        if (configurations.value.length > 0) {
            prefset.setPref("Invocations", gInvocationPrefs);
        }
    } catch(e) {
        log.exception(e);
    }
    return true;
}

// OnPreferencePageClosing(prefset, ok)

// OnPreferencePageCancel(prefset)
//      called when the user clicks cancel
//


function changeLanguage() {
    try {
        savePrefs(gInvocation.currentInstancePreferences);
        var menu = document.getElementById('language');
        gInvocation.currentInstancePreferences.setStringPref("language", menu.selectedItem.label);
        UpdateInvocation();
    } catch(e) {
        log.exception(e);
    }
}

function UpdateInvocation() {
    try {
        // this is where we actually insert our prefs into the panel
        var prefs = gInvocation.currentInstancePreferences;
        var language = prefs.getStringPref("language");

        if (language) {
            // add the debugger component to the ui
            var interpparams = document.getElementById('interpparams');
            if (interpparams.lastChild.nodeName == 'debugOptions')
                interpparams.removeChild(interpparams.lastChild)
            var el = document.createElementNS(XUL_NS, 'debugOptions');
            el.setAttribute('type', language.toLowerCase())
            interpparams.appendChild(el);

        /* XXX for now, we don't want to do this
            var propPanel = document.getElementById('generalProperties');
            if (propPanel.lastChild.nodeName == 'debugProperties')
                propPanel.removeChild(propPanel.lastChild)
            el = document.createElementNS(XUL_NS, 'debugProperties');
            el.setAttribute('type', language.toLowerCase())
            propPanel.appendChild(el);
        */
        }

        document.getElementById('executable-params').value =
                                    prefs.getStringPref("executable-params");
        document.getElementById('filename').value =
                                    prefs.getStringPref("filename");
        document.getElementById('params').value =
                                    prefs.getStringPref("params");
        document.getElementById('cwd').value =
                                    prefs.getStringPref("cwd");
        document.getElementById('debuggerRunInConsole').checked =
                                    prefs.getBooleanPref("use-console");
    } catch(e) {
        log.exception(e);
    }
}

function FillInvocationPopup() {
    try {
        var menupopup = document.getElementById('invocation-configuration-popup');
        while (menupopup.childNodes.length) {
            menupopup.removeChild(menupopup.childNodes[0]);
        }
        var configurations = new Object();
        gInvocationPrefs.getPrefIds(configurations, new Object());
        //dump('got '+configurations.value.length+' invocations for this file\n');
        if (configurations.value.length < 1)
            document.getElementById('deleteButton').setAttribute('disabled','true');
        configurations = configurations.value;
        //dump("current invocation name "+currentInvocation.currentInstancePreferences.id+"\n");
        //currentInvocation.currentInstancePreferences.dump(4);
        //gInvocationPrefs.dump(0);
        for (var i=0; i < configurations.length; i++) {
            var invocation_pref = gInvocationPrefs.getPref(configurations[i]);
            var menuitem = document.createElementNS(XUL_NS, 'menuitem');
            menuitem.setAttribute('label', configurations[i]);
            menupopup.appendChild(menuitem);
            //dump("Adding config "+configurations[i]+"\n");
            //dump("       config "+invocation_pref.id+"\n");
            //invocation_pref.dump(4)
            if (invocation_pref.id == gInvocation.currentInstancePreferences.id) {
                document.getElementById('invocation-configuration').setAttribute('label', configurations[i]);
            }
        }
    } catch(e) {
        log.exception(e);
    }
}

function savePrefs(prefs) {
    if (!prefs.id) {
        //dump("savePrefs has a prefset with no id!\n");
        throw new Error("savePrefs has a prefset with no id!");
    }
    if (document.getElementById('filename').value) {
        prefs.setStringPref("executable-params", document.getElementById('executable-params').value);
        prefs.setStringPref("filename", document.getElementById('filename').value);
        prefs.setStringPref("params", document.getElementById('params').value);
        prefs.setStringPref("cwd", document.getElementById('cwd').value);
        prefs.setBooleanPref("use-console", document.getElementById('debuggerRunInConsole').checked);
        gInvocationPrefs.setPref(prefs.id,prefs);
        //gInvocationPrefs.dump(1);
    }
}

function newPrefset()
{
    // save any settings in the current prefs
    var oldPrefs = gInvocation.currentInstancePreferences;
    savePrefs(oldPrefs);

    // create a new default set
    var catOb = new Object();
    var instOb = new Object();
    gInvocation.getDefaultPreferences(catOb, instOb);
    gInvocation.currentCategoryPreferences = catOb.value;
    gInvocation.currentInstancePreferences = instOb.value;

    // save the prefs we want to carry over
    gInvocation.currentInstancePreferences.update(oldPrefs);
}

function setCurrentPrefs(prefs)
{
    // create a new default set, but use the prefs
    // passed in as the new current instance prefs
    var catOb = new Object();
    var instOb = new Object();
    gInvocation.getDefaultPreferences(catOb, instOb);
    gInvocation.currentCategoryPreferences = catOb.value;
    gInvocation.currentInstancePreferences = prefs;
    gInvocation.currentInstancePreferences.update(prefs);
    gInvocation.currentInstancePreferences.id = prefs.id
}

function ChangeInvocation() {
    // save any settings in the current prefs
    savePrefs(gInvocation.currentInstancePreferences);

    var name = document.getElementById('invocation-configuration').getAttribute("label");
    if (!name) {
        //dump("No name for invocation to change to!\n");
        throw new Error("No name for invocation to change to!");
    }
    var invocation_prefs = gInvocationPrefs.getPref(name);
    invocation_prefs.id = name;
    setCurrentPrefs(invocation_prefs);
    UpdateInvocation();
}

function doSaveNewInvocation() {
    // first, get a name for the new invocation
    try {
        var name = ko.dialogs.prompt("Enter a new configuration name.  The new "+
                                 "configuration will be based on the currently "+
                                 "selected '"+gInvocation.currentInstancePreferences.id+"' configuration.",
                                 "Configuration Name", "",
                                 "Configuration Name");
        if (!name) return;
        if (gInvocationPrefs.hasPref(name)) {
            ko.dialogs.alert("A debugging configuration with the name '"+name+"' already exists.\n");
            return;
        }
        newPrefset();
        gInvocation.currentInstancePreferences.id = name;
        gInvocationPrefs.setPref(gInvocation.currentInstancePreferences.id,
                                 gInvocation.currentInstancePreferences);

        //gInvocation.currentInstancePreferences.dump(0)

        FillInvocationPopup();
        UpdateInvocation();
        document.getElementById('deleteButton').removeAttribute('disabled');
    } catch(e) {
        log.exception(e);
    }
}

function doDeleteInvocation() {
    try {
        var name = document.getElementById('invocation-configuration').getAttribute('label');
        var configurations = new Object();
        gInvocationPrefs.getPrefIds(configurations, new Object());
        if (configurations.value.length < 1) {
            return;
        } else {
            if (ko.dialogs.yesNo("Are you sure you want to delete the "+name+" configuration?") != "Yes") return;
        }
        gInvocationPrefs.deletePref(name);
        gInvocationPrefs.getPrefIds(configurations, new Object());
        var invocation_prefs;
        if (configurations.value.length < 1) {
            // deleted last invocation, start fresh
            createInitialInvocationSet();
            invocation_prefs = gInvocationPrefs.getPref(def_invocation_name);
        } else {
            invocation_prefs = gInvocationPrefs.getPref(configurations.value[0]);
        }
        setCurrentPrefs(invocation_prefs)

        FillInvocationPopup();
        UpdateInvocation();
    } catch(e) {
        log.exception(e);
    }
}

// Utility functions for the various panels.
function doBrowseForDir(widget_id) {
    if (!widget_id)
        widget_id = "cwd";
    var textField = document.getElementById(widget_id);
    var dir = ko.filepicker.getFolder(textField.value);
    if (dir) textField.value = dir;
}

function doBrowseForLanguageFile(widget_id, language) {
    if (typeof(language) == 'undefined') {
        var prefs = gInvocation.currentInstancePreferences;
        language = prefs.getStringPref("language");
    }
    var textField = document.getElementById(widget_id);
    var dir = textField.value;
    if (!dir)
        dir = parent.file.path;
    var path = ko.filepicker.openFile(null, dir,
                                   null, // title
                                   language); // default filter name
    if (path != null) {
        textField.value = path;
    }
}

function doBrowseForXMLInputFile(widget_id) {
    if (!widget_id) return;
    var textField = document.getElementById(widget_id);
    var dir = textField.value;
    if (!dir)
        dir = parent.file.path;
    var path = ko.filepicker.openFile(null, dir,
                                   null, // title
                                   "XML", // default filter name
                                   ["XML", "XSLT", "HTML", "All"]);
    if (path != null) {
        textField.value = path;
    }
}

function doBrowseForCommandFile(widget_id) {
    if (!widget_id) return;
    var textField = document.getElementById(widget_id);
    var dir = textField.value;
    if (!dir)
        dir = parent.file.path;
    var path = ko.filepicker.openExeFile(null, dir);
    if (path != null) {
        textField.value = path;
    }
}


function toggleCGITabs() {
    try {
        var usecgi = document.getElementById("debuggerRunAsCGI");
        if (!usecgi) return;
        var checked = usecgi.getAttribute("checked");
        var t1 = document.getElementById("tab-cgiEnvironment");
        var t2 = document.getElementById("tab-cgiInput");
        if (checked=="true") {
            t1.removeAttribute('hidden');
            t2.removeAttribute('hidden');
        } else {
            t1.setAttribute('hidden','true');
            t2.setAttribute('hidden','true');
        }
    } catch (e) {
        log.exception(e);
    }
}

function updateInterpreterArgumentsWithDir(menuitem) {
    try {
        var dir = ko.filepicker.getFolder();
        if (!dir) return;
        var val = menuitem.getAttribute('value');
        var params = document.getElementById('executable-params');
        if (params.value) {
            params.value += ' '+val+' '+dir;
        } else {
            params.value = val+' '+dir;
        }
        document.getElementById('executable-params').focus();
    } catch (e) {
        log.exception(e);
    }
}

function updateInterpreterArguments(menuitem) {
    try {
        var val = menuitem.getAttribute('value');
        var params = document.getElementById('executable-params');
        if (params.value) {
            params.value += ' '+val;
        } else {
            params.value = val;
        }
        document.getElementById('executable-params').focus();
    } catch (e) {
        log.exception(e);
    }
}
